﻿using Microsoft.Phone.Globalization;
using System.Collections.Generic;
using System.Globalization;

namespace Coding4Fun.MissionControl.WP.Misc
{
    public class Group<T> : List<T>
    {
        public delegate string GetKeyDelegate(T item);

        public string Key { get; private set; }

        public Group(string key)
        {
            Key = key;
        }

        public static List<Group<T>> CreateGroups(SortedLocaleGrouping slg)
        {
            List<Group<T>> list = new List<Group<T>>();
            foreach (string key in slg.GroupDisplayNames)
            {
                list.Add(new Group<T>(key));
            }

            return list;
        }

        public static List<Group<T>> CreateGroups(IEnumerable<T> items, CultureInfo ci, GetKeyDelegate getKey,
            bool sort)
        {
            SortedLocaleGrouping slg = new SortedLocaleGrouping(ci);
            List<Group<T>> list = CreateGroups(slg);

            foreach (T item in items)
            {
                int index = 0;
                index = slg.GetGroupIndex(getKey(item));
                if (index >= 0 && index < list.Count)
                {
                    list[index].Add(item);
                }
            }

            if (sort)
            {
                foreach (Group<T> group in list)
                {
                    group.Sort((c0, c1) => { return ci.CompareInfo.Compare(getKey(c0), getKey(c1)); });
                }
            }

            return list;
        }
    }
}
